#region license
// Copyright (c) 2004, Rodrigo B. de Oliveira (rbo@acm.org)
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//     * Neither the name of Rodrigo B. de Oliveira nor the names of its
//     contributors may be used to endorse or promote products derived from this
//     software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#endregion

namespace Boo.Lang.Compiler.Pipelines
{
	using System;
	using System.IO;
	using System.Reflection;
	using Boo.Lang.Compiler.Util;

	public class Parse : CompilerPipeline
	{
		static Type _defaultParserStepType;
		
		public static ICompilerStep NewParserStep()
		{
			if (null == _defaultParserStepType)
			{
				_defaultParserStepType = FindParserAssembly().GetType("Boo.Lang.Parser.BooParsingStep", true);
			}
			return (ICompilerStep)Activator.CreateInstance(_defaultParserStepType);
		}

		static Assembly FindParserAssembly()
		{
			Assembly thisAssembly = typeof(Parse).Assembly;
			string thisLocation = Permissions.HasDiscoveryPermission ?  thisAssembly.Location : "";
			string parserLocation = thisLocation.EndsWith("Boo.Lang.Compiler.dll")
                	? thisLocation.Substring(0, thisLocation.Length - "Boo.Lang.Compiler.dll".Length) + "Boo.Lang.Parser.dll"
                	: "";

			return File.Exists(parserLocation)
					? Assembly.LoadFrom(parserLocation)
					: Assembly.Load(thisAssembly.FullName.Replace("Boo.Lang.Compiler", "Boo.Lang.Parser"));
		}
		
		public Parse()
		{
			Add(NewParserStep());
		}
	}
}
